require 'json'
require 'bunny'

module RabbitExample
  class Connection
    def initialize(ws)
      @ws = ws
    end

    def start!
      @sampled_uri = amqp_credentials['uris'].sample || amqp_credentials['uri']

      connection.start
      index = amqp_credentials['uris'].index(@sampled_uri)
      ws_puts "<b><font color='DarkMagenta'>Connected to #{@sampled_uri} (index #{index})</font></b><br/>\n"
    rescue => e
      puts_error "[ERROR] Connection #{@sampled_uri} failed to start: #{e.message}."
      clear_connection_state
      unless closed?
        sleep 2
        retry
      end
    end

    def stop!
      connection.stop
      clear_connection_state
    rescue => e
      puts_error "[ERROR] Connection #{@sampled_uri} failed to stop: #{e.message}."
    end

    def restart!
      puts_warning 'Trying to reconnect'
      stop!
      start!
    end

    def closed?
      ws.state == :closed
    end

    private

    attr_reader :ws

    def clear_connection_state
      @connection = @channel = @queue = nil
    end

    def queue
      @queue ||= channel.queue(queue_name, durable: true)
    end

    def channel
      @channel ||= connection.create_channel
    end

    def connection
      @connection ||= Bunny.new(
        @sampled_uri,
        tls_cert: ENV['CF_INSTANCE_CERT'],
        tls_key: ENV['CF_INSTANCE_KEY'],
        verify_peer: false)
    end

    def vcap_services
      @vcap_services ||= JSON.parse(ENV['VCAP_SERVICES'])
    end

    def queue_name
      ENV['QUEUE_NAME'] || 'testq'
    end

    def amqp_credentials
      protocols = extract_credentials(vcap_services)['protocols']
      protocols['amqp+ssl'] || protocols['amqp']
    end

    def extract_credentials(vcap_services)
      vcap_services.each do |service_key, service|
        service.each do |element|
          return element["credentials"] if element["tags"].include?("rabbitmq")
        end
      end

      raise "no rabbitmq tag found!"
    end

    def puts_success(msg)
      ws_puts "#{msg} <br />\n"
    end

    def puts_error(msg)
      ws_puts "<font color='red'>#{msg}</font><br />\n"
    end

    def puts_warning(msg)
      ws_puts "<font color='orange'>#{msg}</font><br />\n"
    end

    def ws_puts(msg)
      ws.send msg
      STDOUT.puts msg
    end
  end
end
